﻿// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -------------------------------------------------------------------
// FVX Sample Program
// 
//
// 作成者 : R.Nagashima
// 履歴   : 2007/01/17 初版
//
//-------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using FVDATASLib;


namespace BinCalc
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        //処理範囲
        public FvRectangle m_ExeRegion = new FvRectangle();
        // ビデオデバイスがオープンされているか
        private bool m_blnVideoOpened = false;
        // ビデオ設定ファイル名（フルパス）
        private string m_strVideoConfFilePath = "";
        // 初期ディレクトリ
        private string m_strDirectory;

        private int m_intMasterBinLevel = 128;
        private int m_intSrcBinLevel = 128;

        // axFvxImageMaster     :マスタ濃淡画像
    	// axFvxImageMasterBin  :FvxImageMasterを２値化したもの
    	// axFvxImageSrc        :処理対象濃淡画像
    	// axFvxImageSrcBin     :FvxImageSrcを２値化したもの
    	// axFvxImageDstBin     :処理結果の２値画像
    	// axFvxImageTemp       :ファイル読込時に使用



        private void frmMain_Load(object sender, EventArgs e)
        {
            int IntErr;
            frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
            frmLicense fLicense = new frmLicense();

            // 変数初期化
            m_blnVideoOpened = false;
            btnGrab.Enabled = false;

            //画面の生成
            axFvxDisplayEx1.CreateDisplay();

            //処理範囲の初期値設定(画像の全面に設定）
            m_ExeRegion.xs = -1;
            m_ExeRegion.xe = -1;
            m_ExeRegion.ys = -1;
            m_ExeRegion.ye = -1;

            //作業ディレクトリ退避
            m_strDirectory = Directory.GetCurrentDirectory();

            //使用許諾の確認
            if (fLicense.ShowDialog() == DialogResult.OK)
            {
                //ビデオ設定ファイルの選択
                if (fVideoConf.ShowDialog() == DialogResult.OK)
                    m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

                if (fVideoConf.chkNotUseCamera.Checked == false)
                {
                    //ビデオ設定ファイルをコンポーネントに設定
                    axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

                    //デバイスオープン
                    try
                    {
                        axFvxVideo1.OpenDevice();
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
                    //本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
                    //お使いのキャプチャボードに合わせて変更してください。
                    Thread.Sleep(350);

                    //GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
                    //あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
                    //（取り込みサイズとImageのサイズが違う場合エラーになります）
                    axFvxVideo1.ChangeImageSize(axFvxImageMaster.GetOcx());//マスタ画像
                    axFvxVideo1.ChangeImageSize(axFvxImageSrc.GetOcx());//処理対象画像
                    try
                    {
                        // 同期取込
                        axFvxVideo1.GrabSync(axFvxImageMaster.GetOcx());
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //ビデオデバイスの状態を保存
                    m_blnVideoOpened = true;
                    btnGrab.Enabled = true;

                    //画像2値化して表示
                    axFvxChangeImage1.FixedLevel(axFvxImageMaster.GetOcx(), axFvxImageMasterBin.GetOcx(), 128);
                    axFvxDisplayEx1.DrawImage(axFvxImageMasterBin.GetOcx());
                    axFvxChangeImage1.FixedLevel(axFvxImageSrc.GetOcx(), axFvxImageSrcBin.GetOcx(), 128);
                }

                radioDispMaster.Checked = true;

            }
            else
            {
                //終了
                Application.Exit();
                return;
            }
        }

        //終了時
        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (m_blnVideoOpened)
                axFvxVideo1.CloseDevice();
        }

        //カメラ取り込み
        private void btnGrab_Click(object sender, EventArgs e)
        {
            //画像ファイル読み込みによって、画像サイズが初期化時から
            //変更されている可能性があるため、ChangeImageSizeを行います。
            //画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
            int BinLevel = Convert.ToInt32(txtBinLevel.Text);

            axFvxVideo1.ChangeImageSize(axFvxImageMaster.GetOcx());
            axFvxVideo1.ChangeImageSize(axFvxImageSrc.GetOcx());

            try
            {
                if (radioDispMaster.Checked == true)
                {
                    axFvxVideo1.GrabSync(axFvxImageMaster.GetOcx());
                    axFvxChangeImage1.FixedLevel(axFvxImageMaster.GetOcx(), axFvxImageMasterBin.GetOcx(), BinLevel);
                    axFvxDisplayEx1.DrawImage(axFvxImageMasterBin.GetOcx());
                }
                else if (radioDispSrc.Checked == true)
                {
                    axFvxVideo1.GrabSync(axFvxImageSrc.GetOcx());
                    axFvxChangeImage1.FixedLevel(axFvxImageSrc.GetOcx(), axFvxImageSrcBin.GetOcx(), BinLevel);
                    axFvxDisplayEx1.DrawImage(axFvxImageSrcBin.GetOcx());
                }
            }
            catch (Exception)
            {
                int IntErr;
                axFvxVideo1.GetErrReport(out IntErr);
                MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
            }
        }

        //画像ファイル読込
        private void btnLoadImage_Click(object sender, EventArgs e)
        {
            // ダイアログにて画像ファイルを選択します｡
            openFileDialog1.InitialDirectory = m_strDirectory;
            openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            openFileDialog1.DefaultExt = "*.bmp";
            openFileDialog1.ShowDialog();

            int BinLevel = Convert.ToInt32(txtBinLevel.Text);

            if (openFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを用いて、
                //ファイルからImageに画像を読み込みます｡
                axFvxFileAccess1.FilePath = openFileDialog1.FileName;

                try
                {
                    axFvxFileAccess1.LoadBitmap(axFvxImageTemp.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
                    return;
                }

                //画像タイプチェック
                if (axFvxImageTemp.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray || axFvxImageTemp.ImageWidth != 512 || axFvxImageTemp.ImageHeight != 480)
                {
                    MessageBox.Show("512*480 1/8bitグレイの画像ではありません｡", "error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    axFvxImageTemp.BitPerPixel = 8;
                    axFvxImageTemp.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
                    axFvxImageTemp.ImageWidth = 512;
                    axFvxImageTemp.ImageHeight = 480;
                }

                //2値か濃淡か
                if (axFvxImageTemp.BitPerPixel == 1)
                {
                    if (radioDispMaster.Checked == true)
                    {
                        axFvxImageMaster.ClearImage();
                        axFvxImageTemp.CopyTo(axFvxImageMasterBin.GetOcx());
                        axFvxDisplayEx1.DrawImage(axFvxImageMasterBin.GetOcx());
                    }
                    else if (radioDispSrc.Checked == true)
                    {
                        axFvxImageSrc.ClearImage();
                        axFvxImageTemp.CopyTo(axFvxImageSrcBin.GetOcx());
                        axFvxDisplayEx1.DrawImage(axFvxImageSrcBin.GetOcx());
                    }
                }
                else if (axFvxImageTemp.BitPerPixel == 8)
                {
                    if (radioDispMaster.Checked == true)
                    {
                        axFvxImageTemp.CopyTo(axFvxImageMaster.GetOcx());
                        axFvxChangeImage1.FixedLevel(axFvxImageMaster,axFvxImageMasterBin,m_intMasterBinLevel);
                        axFvxDisplayEx1.DrawImage(axFvxImageMasterBin.GetOcx());
                    }
                    else if (radioDispSrc.Checked == true)
                    {
                        axFvxImageTemp.CopyTo(axFvxImageSrc.GetOcx());
                        axFvxChangeImage1.FixedLevel(axFvxImageSrc, axFvxImageSrcBin, m_intSrcBinLevel);
                        axFvxDisplayEx1.DrawImage(axFvxImageSrcBin.GetOcx());
                    }
                }         
                else
                    MessageBox.Show("1/8bit以外のbmpです", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }  

        //処理範囲設定
        private void btnExeRegion_Click(object sender, EventArgs e)
        {
            int IntOK = 0;
            double dblTemp = 0.0;

            axFvxROI1.SetRectangle(m_ExeRegion);

            // FvxROIを使って処理範囲選択ダイアログを出します｡
            axFvxROI1.UsePoint = false;
            axFvxROI1.MakeROIDialog(axFvxImageMasterBin.GetOcx(), out IntOK);

            if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
            {
                axFvxROI1.GetRectangle(m_ExeRegion);
                // 始点と終点が逆になっていないかチェックします｡
                // FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
                // 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
                if (m_ExeRegion.xe < m_ExeRegion.xs)
                {
                    dblTemp = m_ExeRegion.xs;
                    m_ExeRegion.xs = m_ExeRegion.xe;
                    m_ExeRegion.xe = dblTemp;
                }
                if (m_ExeRegion.ye < m_ExeRegion.ys)
                {
                    dblTemp = m_ExeRegion.ys;
                    m_ExeRegion.ys = m_ExeRegion.ye;
                    m_ExeRegion.ye = dblTemp;
                }

                axFvxBinFilter1.SetInspectArea(m_ExeRegion);
                axFvxDisplayEx1.ClearOverlay(-1);
                axFvxDisplayEx1.PenColor = Color.Blue;
                axFvxDisplayEx1.DrawData(0, m_ExeRegion);
            }
        }

        //画像保存
        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            saveFileDialog1.InitialDirectory = m_strDirectory;
            saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            saveFileDialog1.DefaultExt = "*.bmp";
            saveFileDialog1.ShowDialog();

            if (saveFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを使って
                //Imageをファイルに保存します｡
                axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
                try
                {
                    if(radioDispMaster.Checked == true) axFvxFileAccess1.SaveBitmap(axFvxImageMasterBin.GetOcx());
                    if(radioDispSrc.Checked == true) axFvxFileAccess1.SaveBitmap(axFvxImageSrcBin.GetOcx());
                    if (radioDispDst.Checked == true) axFvxFileAccess1.SaveBitmap(axFvxImageDstBin.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
                    return;
                }
            }
        }
            
        //And
        private void btnAnd_Click(object sender, EventArgs e)
        {
            radioDispDst.Checked = false;
            axFvxBinFilter1.And(axFvxImageMasterBin.GetOcx(), axFvxImageSrcBin.GetOcx(), axFvxImageDstBin.GetOcx());
            radioDispDst.Checked = true;
        }

        //Eor
        private void btnEOR_Click(object sender, EventArgs e)
        {
            radioDispDst.Checked = false;
            axFvxBinFilter1.Eor(axFvxImageMasterBin.GetOcx(), axFvxImageSrcBin.GetOcx(), axFvxImageDstBin.GetOcx());
            radioDispDst.Checked = true;
        }

        //Or
        private void btnOr_Click(object sender, EventArgs e)
        {
            radioDispDst.Checked = false;
            axFvxBinFilter1.Or(axFvxImageMasterBin.GetOcx(), axFvxImageSrcBin.GetOcx(), axFvxImageDstBin.GetOcx());
            radioDispDst.Checked = true;
        }     

        //マスタ画像表示
        private void radioDispMaster_CheckedChanged(object sender, EventArgs e)
        {
            axFvxDisplayEx1.DrawImage(axFvxImageMasterBin.GetOcx());
            txtBinLevel.Text = m_intMasterBinLevel.ToString();
            btnLoadImage.Enabled = true;
            btnExeRegion.Enabled = true;
            if (m_blnVideoOpened == true) btnGrab.Enabled = true;
        }

        //処理対象画像表示
        private void radioDispSrc_CheckedChanged(object sender, EventArgs e)
        {
            axFvxDisplayEx1.DrawImage(axFvxImageSrcBin.GetOcx());
            txtBinLevel.Text = m_intSrcBinLevel.ToString();
            btnLoadImage.Enabled = true;
            btnExeRegion.Enabled = true;
            if (m_blnVideoOpened == true) btnGrab.Enabled = true;
        }

        //結果画像表示
        private void radioDispDst_CheckedChanged(object sender, EventArgs e)
        {
            axFvxDisplayEx1.DrawImage(axFvxImageDstBin.GetOcx());
            btnLoadImage.Enabled = false;
            btnExeRegion.Enabled = false;
            btnGrab.Enabled = false;
        }


        //2値閾値変更
        private void txtBinLevel_TextChanged(object sender, EventArgs e)
        {
            int length = txtBinLevel.Text.Length;

            if (length != 0)
            {
				int BinLevel;
				int.TryParse(txtBinLevel.Text,out BinLevel);

                if (BinLevel < 0)
                {
                    BinLevel = 0;
                    txtBinLevel.Text = "0";
                }
				else if (BinLevel > 255)
				{
					BinLevel = 255;
					txtBinLevel.Text = "255";
				}

                if (radioDispMaster.Checked == true)
                {
                    m_intMasterBinLevel = BinLevel;
                    axFvxChangeImage1.FixedLevel(axFvxImageMaster.GetOcx(), axFvxImageMasterBin.GetOcx(), m_intMasterBinLevel);
                    axFvxDisplayEx1.DrawImage(axFvxImageMasterBin.GetOcx());
                }
                else if (radioDispSrc.Checked == true)
                {
                    m_intSrcBinLevel = BinLevel;
                    axFvxChangeImage1.FixedLevel(axFvxImageSrc.GetOcx(), axFvxImageSrcBin.GetOcx(), m_intSrcBinLevel);
                    axFvxDisplayEx1.DrawImage(axFvxImageSrcBin.GetOcx());
                }
            }
        }

        private void txtBinLevel_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;
        }
    }
}